//Defining the names for the tree node leaf
const Data = {
  "name": "Animal",
  "value": 30,
  "type": "gray",
  "level": "yellow",
  "children": [
    {
      "name": "Mammal",
      "value": 15,
      "type": "yellow",
      "level": "green",
      "children": [
        {
          "name": "Primates",
          "value": 6,
          "type": "black",
          "level": "lightblue"
        },
       
        {
          "name": "Carnivores",
          "value": 6,
          "type": "red",
          "level": "brown"
        }
      ]
    },
    {
      "name": "Reptile ",
      "value": 15,
      "type": "orange",
      "level": "lightgreen",
       "children": [
        {
          "name": "Snakes",
          "value": 6,
          "type": "red",
          "level": "black"
        },
         {
          "name": "Lizards",
          "value": 6,
          "type": "black",
          "level": "orange"
        }
       ]
    },
    {
      "name": "Bird",
      "value": 15,
      "type": "Purple",
      "level": "indigo",
      "children": [
        {
          "name": "wood pecker",
          "value": 6,
          "type": "red",
          "level": "yellow"
        },
        
        {
          "name": "crow",
          "value": 6,
          "type": "green",
          "level": "red"
        }
        
      ]
    },
  ]
};

// Set the tree's border, width, and height.
const margin = {top: 30, right: 220, bottom: 41, left: 200},
      width  = 850 - margin.left - margin.right,
      height = 650 - margin.top - margin.bottom;

// Declares the height and width of a tree.
const tree = d3.tree().size([height, width]);

//Using relationships to classify data and allocate it to a hierarchy.
let nodes = d3.hierarchy(Data, node => node.children);

// connects the node data to its corresponding location in the tree.
nodes = tree(nodes);

// add an svg to the body
const svg = d3.select("body").append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom),
      g = svg.append("g")//appends group
        .attr("transform",
            "translate(" + margin.left + "," + margin.top + ")");

// building bridges or connections between nodes
const link = g.selectAll(".link")
    .data( nodes.descendants().slice(1))
  .enter().append("path")
    .attr("class", "link")
    .style("stroke", d => d.data.level)
    .attr("d", d => {
       return "M" + d.y + "," + d.x
         + "C" + (d.y + d.parent.y) / 2 + "," + d.x
         + " " + (d.y + d.parent.y) / 2 + "," + d.parent.x
         + " " + d.parent.y + "," + d.parent.x;
       });

// creating a group for each node
const node = g.selectAll(".node")
    .data(nodes.descendants())
    .enter().append("g")
    .attr("class", d => "node" + (d.children ? " node--internal" : " node--leaf"))
    .attr("transform", d => "translate(" + d.y + "," + d.x + ")");

// gives the node a rounded shape
node.append("circle")
  .attr("r", d => d.data.value)
  .style("stroke", d => d.data.type)
  .style("fill", d => d.data.level);
  
// adding a new node to the list
node.append("text")
  .attr("dy", ".60em")
  .attr("x", d => d.children ? (d.data.value + 5) * -1 : d.data.value + 5)
  .attr("y", d => d.children && d.depth !== 0 ? -(d.data.value + 5) : d)
  .style("text-anchor", d => d.children ? "end" : "start")
  .text(d => d.data.name);